@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-from django.conf import settings
 from django.core import exceptions, validators
 from django.db import models
 from django.utils import six
@@ -17,11 +16,21 @@ DEFAULT_DELIMITER = ','
 
 
 @python_2_unicode_compatible
-class SelectMultipleField(models.Field):
+class SelectMultipleField(six.with_metaclass(models.SubfieldBase,
+                                             models.Field)):
     """Stores multiple selection choices as serialized list"""
 
+    default_error_messages = {
+        'blank': _("This field cannot be blank."),
+        'invalid_type': _(
+            "Types passed as value must be string, list, tuple or None, "
+            "not '%(value)s'."),
+        'invalid_choice': _(
+            "Select a valid choice. %(value)s is not one of the available "
+            "choices."),
+        'null': _("This field cannot be null."),
+    }
     description = _('Select multiple field')
-    __metaclass__ = models.SubfieldBase
 
     def __init__(self, *args, **kwargs):
         """
@@ -42,19 +51,30 @@ class SelectMultipleField(models.Field):
         """
         When SelectMultipleField is assigned a value, this method coerces
         into a list usable by Python
+
+        value is Encoded strings from the database or Python native types in
+        need of validation
+
+        Raises ValidationError if value is not in choices or if invalid type
+
+        Returns list
         """
-        delimiter = getattr(
-            settings, 'SELECTMULTIPLEFIELD_DELIMITER', DEFAULT_DELIMITER)
-        if isinstance(value, six.string_types):
-            pyval = decode_csv_to_list(value)
-            return pyval
-        elif value is None:
-            return value
-        if isinstance(value, (list, tuple)):
+        if value is None:
             return value
 
-        raise Exception('End of Model.to_python')
-        return delimiter.join(value)
+        elif isinstance(value, (list, tuple)):
+            self.validate_options_list(value)
+            return value
+
+        elif isinstance(value, six.string_types):
+            #
+            # Strings are always encoded choices
+            #
+            native = decode_csv_to_list(value)
+            return native
+
+        msg = self.error_messages['invalid_type'] % {'value': type(value)}
+        raise exceptions.ValidationError(msg)
 
     def get_prep_value(self, value):
         """
@@ -62,25 +82,25 @@ class SelectMultipleField(models.Field):
 
         This takes a Python list and encodes it into a form storable in the
         database
-        """
-        return encode_list_to_csv(value)
 
-    def value_from_object(self, obj):
+        Returns a string or None
         """
-        Returns the value of this field in the given model instance.
-        """
-        value = getattr(obj, self.attname)
-        return value
+        if value is None:
+            return None
+
+        return encode_list_to_csv(value)
 
     def get_choices(self, **kwargs):
         """
-        Choices without initial blank choices
+        Choices from model without initial blank choices
+
+        ie Stop widget from producing <option value="">---------</option>
         """
         field_options = {
             'include_blank': False
         }
-        kwargs.update(field_options)
-        return super(SelectMultipleField, self).get_choices(**kwargs)
+        field_options.update(kwargs)
+        return super(SelectMultipleField, self).get_choices(**field_options)
 
     def validate(self, value, model_instance):
         """
@@ -90,6 +110,7 @@ class SelectMultipleField(models.Field):
         if not self.editable:
             # Skip validation for non-editable fields.
             return
+
         if self._choices and value:
             if isinstance(value, (list, tuple)):
                 bad_values = []
@@ -98,17 +119,11 @@ class SelectMultipleField(models.Field):
                         bad_values.append(opt)
                 if len(bad_values) == 0:
                     return
+                else:
+                    msg = self.error_messages['invalid_choice'] % {'value': bad_values}
+                    raise exceptions.ValidationError(msg)
 
-            for option_key, option_value in self.choices:
-                if isinstance(option_value, (list, tuple)):
-                    # This is an optgroup, so look inside the group for
-                    # options.
-                    for optgroup_key, optgroup_value in option_value:
-                        if value == optgroup_key:
-                            return
-                elif value == option_key:
-                    return
-            msg = self.error_messages['invalid_choice'] % value
+            msg = self.error_messages['invalid_choice'] % {'value': value}
             raise exceptions.ValidationError(msg)
 
         if value is None and not self.null:
@@ -117,9 +132,33 @@ class SelectMultipleField(models.Field):
         if not self.blank and value in validators.EMPTY_VALUES:
             raise exceptions.ValidationError(self.error_messages['blank'])
 
+    def validate_options_list(self, value):
+        """
+        Checks that all options in value list are in choices
+
+        Raises ValidationError if an option in value list is not in choices
+
+        Returns None if all values are in choices
+        """
+        for option in value:
+            if not self.validate_option(option):
+                msg = self.error_messages['invalid_choice'] % {'value': option}
+                raise exceptions.ValidationError(msg)
+
+        return
+
+    def validate_option(self, value):
+        """
+        Checks that value is in choices
+        """
+        choices = dict(self.get_choices())
+        return value in choices.keys()
+
     def formfield(self, **kwargs):
         """
-        This returns the right formclass without calling super
+        This returns the correct formclass without calling super
+
+        Returns select_multiple_field.forms.SelectMultipleFormField
         """
         defaults = {'required': not self.blank,
                     'label': capfirst(self.verbose_name),
@@ -130,10 +169,12 @@ class SelectMultipleField(models.Field):
                 defaults['show_hidden_initial'] = True
             else:
                 defaults['initial'] = self.get_default()
+
         if self.choices:
-            # Fields with choices get special treatment.
-            include_blank = (self.blank or
-                             not (self.has_default() or 'initial' in kwargs))
+            # Django normally includes an empty choice if blank, has_default
+            # and initial are all False, we are intentially breaking this
+            # convention
+            include_blank = self.blank
             defaults['choices'] = self.get_choices(include_blank=include_blank)
             defaults['coerce'] = self.to_python
             if self.null:
@@ -141,11 +182,12 @@ class SelectMultipleField(models.Field):
 
             # Many of the subclass-specific formfield arguments (min_value,
             # max_value) don't apply for choice fields, so be sure to only pass
-            # the values that TypedChoiceField will understand.
+            # the values that SelectMultipleFormField will understand.
             for k in kwargs.keys():
                 if k not in ('coerce', 'empty_value', 'choices', 'required',
                              'widget', 'label', 'initial', 'help_text',
                              'error_messages', 'show_hidden_initial'):
                     del kwargs[k]
+
         defaults.update(kwargs)
         return forms.SelectMultipleFormField(**defaults)
