@@ -3,15 +3,33 @@ from __future__ import unicode_literals
 
 from django.core.urlresolvers import reverse
 from django.test import SimpleTestCase, TestCase
+from django.utils.datastructures import MultiValueDict
+from django.utils.http import urlencode
 
 from .models import Pizza, show_topping
 
 
 class PizzaListViewTestCase(TestCase):
 
-    def test_view(self):
+    def test_no_pizzas(self):
+        p = Pizza.objects.all()
+        self.assertEqual(len(p), 0, 'Test requires no pizzas')
         response = self.client.get(reverse('pizza:list'))
         self.assertEqual(response.status_code, 200)
+        self.assertTrue('No pizzas found' in response.content.decode('utf-8'))
+
+    def test_many_pizzas(self):
+        NUM_PIZZAS = 30
+        pizzas = []
+        for n in range(NUM_PIZZAS):
+            p = Pizza.objects.create(toppings=[Pizza.PEPPERONI])
+            pizzas.append(p)
+
+        self.assertEqual(len(pizzas), NUM_PIZZAS, 'Test requires pizzas')
+        response = self.client.get(reverse('pizza:list'))
+        self.assertEqual(response.status_code, 200)
+        self.assertTrue(
+            show_topping(Pizza.PEPPERONI) in response.content.decode('utf-8'))
 
 
 class PizzaCreateViewTestCase(TestCase):
@@ -20,24 +38,45 @@ class PizzaCreateViewTestCase(TestCase):
         response = self.client.get(reverse('pizza:create'))
         self.assertEqual(response.status_code, 200)
 
-    def test_creation(self):
+    def test_creation_single(self):
         data = {
-            'toppings': [Pizza.MOZZARELLA, Pizza.PEPPERONI]
+            'toppings': [Pizza.BLACK_OLIVES]
         }
-        response = self.client.post(reverse('pizza:create'), data)
+        response = self.client.post(
+            reverse('pizza:create'),
+            urlencode(MultiValueDict(data), doseq=True),
+            content_type='application/x-www-form-urlencoded'
+            )
+
         self.assertEqual(response.status_code, 302)
         self.assertRedirects(
             response,
             'http://testserver' + reverse('pizza:created'))
         p = Pizza.objects.all()[0]
-        self.assertTrue(Pizza.MOZZARELLA in p.toppings)
-        self.assertTrue(Pizza.PEPPERONI in p.toppings)
+        self.assertIn(Pizza.BLACK_OLIVES, p.toppings)
+
+    def test_creation_multiple(self):
+        data = {
+            'toppings': [Pizza.MOZZARELLA, Pizza.PANCETTA]
+        }
+        response = self.client.post(
+            reverse('pizza:create'),
+            urlencode(MultiValueDict(data), doseq=True),
+            content_type='application/x-www-form-urlencoded'
+            )
+        self.assertEqual(response.status_code, 302)
+        self.assertRedirects(
+            response,
+            'http://testserver' + reverse('pizza:created'))
+        p = Pizza.objects.all()[0]
+        self.assertIn(Pizza.MOZZARELLA, p.toppings)
+        self.assertIn(Pizza.PANCETTA, p.toppings)
 
 
 class PizzaDetailViewTestCase(TestCase):
 
     def setUp(self):
-        self.pizza = Pizza(toppings=[Pizza.PEPPERONI])
+        self.pizza = Pizza(toppings=[Pizza.EGG])
         self.pizza.save()
 
     def test_view(self):
@@ -50,29 +89,31 @@ class PizzaDetailViewTestCase(TestCase):
 class PizzaUpdateViewTestCase(TestCase):
 
     def setUp(self):
-        self.pizza = Pizza(toppings=[Pizza.MOZZARELLA, Pizza.PEPPERONI])
+        self.pizza = Pizza(toppings=[Pizza.MUSHROOMS, Pizza.TOMATO])
         self.pizza.save()
 
     def test_change_toppings(self):
         data = {
-            'toppings': [Pizza.CHEDDAR_CHEESE, Pizza.MOZZARELLA]
+            'toppings': [Pizza.CHEDDAR_CHEESE, Pizza.MUSHROOMS]
         }
         response = self.client.post(
-            reverse('pizza:update', args=[self.pizza.id]), data)
+            reverse('pizza:update', args=[self.pizza.id]),
+            urlencode(MultiValueDict(data), doseq=True),
+            content_type='application/x-www-form-urlencoded')
         self.assertEqual(response.status_code, 302)
         self.assertRedirects(
             response,
             'http://testserver' + reverse('pizza:updated'))
         p = Pizza.objects.all()[0]
         self.assertTrue(Pizza.CHEDDAR_CHEESE in p.toppings)
-        self.assertTrue(Pizza.MOZZARELLA in p.toppings)
-        self.assertFalse(Pizza.PEPPERONI in p.toppings)
+        self.assertTrue(Pizza.MUSHROOMS in p.toppings)
+        self.assertFalse(Pizza.TOMATO in p.toppings)
 
 
 class PizzaDeleteViewTestCase(TestCase):
 
     def setUp(self):
-        self.pizza = Pizza(toppings=[Pizza.PEPPERONI])
+        self.pizza = Pizza(toppings=[Pizza.PROSCIUTTO_CRUDO])
         self.pizza.save()
 
     def test_delete_pizza(self):
